@@ -36,6 +36,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 Components.utils.import("resource://app/modules/editorHelper.jsm");
+Components.utils.import("resource://app/modules/urlHelper.jsm");
 
 const kBASE_COMMAND_CONTROLLER_CID = "@mozilla.org/embedcomp/base-command-controller;1";
 
@@ -84,7 +85,6 @@ var ComposerCommands = {
 
   goUpdateComposerMenuItems: function goUpdateComposerMenuItems(commandset)
   {
-    dump("Updating commands for " + commandset.id + "\n");
     for (var i = 0; i < commandset.childNodes.length; i++)
     {
       var commandNode = commandset.childNodes[i];
@@ -129,15 +129,15 @@ var ComposerCommands = {
         case "cmd_em":
         case "cmd_code":
         case "cmd_strikethrough":
-	      case "cmd_superscript":
-	      case "cmd_subscript":
-	      case "cmd_nobreak":
-	      case "cmd_var":
-	      case "cmd_samp":
-	      case "cmd_code":
-	      case "cmd_acronym":
-	      case "cmd_abbr":
-	      case "cmd_cite":
+        case "cmd_superscript":
+        case "cmd_subscript":
+        case "cmd_nobreak":
+        case "cmd_var":
+        case "cmd_samp":
+        case "cmd_code":
+        case "cmd_acronym":
+        case "cmd_abbr":
+        case "cmd_cite":
         case "cmd_tt":
 
         case "cmd_ul":
@@ -161,10 +161,10 @@ var ComposerCommands = {
         case "cmd_outdent":
           break;
 
-        default: dump("no update for command: " +command+"\n");
+        default: break;
       }
     }
-    catch (e) { dump("An error occurred updating the "+command+" command: \n"+e+"\n"); }
+    catch (e) {  }
   },
 
   pokeStyleUI: function pokeStyleUI(uiID, aDesiredState)
@@ -184,7 +184,7 @@ var ComposerCommands = {
         newState = "false";
       commandNode.setAttribute("state", newState);
     }
-   } catch(e) { dump("poking UI for "+uiID+" failed: "+e+"\n"); }
+   } catch(e) {  }
   },
 
   newCommandParams: function newCommandParams()
@@ -192,7 +192,7 @@ var ComposerCommands = {
     try {
       return Components.classes["@mozilla.org/embedcomp/command-params;1"].createInstance(Components.interfaces.nsICommandParams);
     }
-    catch(e) { dump("error thrown in newCommandParams: "+e+"\n"); }
+    catch(e) {  }
     return null;
   },
 
@@ -244,7 +244,7 @@ var ComposerCommands = {
 
       this.pokeMultiStateUI(commandID, cmdParams);
 
-    } catch(e) { dump("error thrown in doStatefulCommand: "+e+"\n"); }
+    } catch(e) {  }
   },
 
   doCommandWithValue: function doCommandWithValueFromAttribute(commandID, aValue)
@@ -259,7 +259,7 @@ var ComposerCommands = {
 
       this.pokeMultiStateUI(commandID, cmdParams);
 
-    } catch(e) { dump("error thrown in doStatefulCommand: "+e+"\n"); }
+    } catch(e) { }
   },
 
   goDoCommandParams: function goDoCommandParams(command, params)
@@ -283,10 +283,7 @@ var ComposerCommands = {
         }
       }
     }
-    catch (e)
-    {
-      dump("An error occurred executing the "+command+" command\n");
-    }
+    catch (e) { }
   },
 
   setupMainCommands: function setupMainCommands()
@@ -295,9 +292,7 @@ var ComposerCommands = {
     if (!commandTable)
       return;
     
-    //dump("Registering plain text editor commands\n");
     commandTable.registerCommand("cmd_stopLoading", cmdStopLoading);
-    //commandTable.registerCommand("cmd_copy",        cmdCopy);
     commandTable.registerCommand("cmd_open",        cmdOpen);
     commandTable.registerCommand("cmd_save",        cmdSave);
     commandTable.registerCommand("cmd_saveAs",      cmdSaveAs);
@@ -305,12 +300,36 @@ var ComposerCommands = {
     commandTable.registerCommand("cmd_fullScreen",  cmdFullScreen);
     commandTable.registerCommand("cmd_new",         cmdNew);
     commandTable.registerCommand("cmd_newWizard",   cmdNewWizard);
-    commandTable.registerCommand("cmd_renderedHTMLEnabler",  cmdDummyHTML);
-    commandTable.registerCommand("cmd_viewModeEnabler", cmdDummy2HTML);
+    commandTable.registerCommand("cmd_renderedHTMLEnabler",    cmdDummyHTML);
+    commandTable.registerCommand("cmd_renderedSourceEnabler",  cmdDummySource);
+    commandTable.registerCommand("cmd_renderedAllEnabler",     cmdDummyAll);
+    commandTable.registerCommand("cmd_viewModeEnabler", cmdViewModeEnabler);
     commandTable.registerCommand("cmd_cleanup",     cmdMarkupCleaner);
     commandTable.registerCommand("cmd_browse",      cmdBrowseCommand);
 
-    commandTable.registerCommand("cmd_table",       cmdInsertOrEditTableCommand);
+    commandTable.registerCommand("cmd_table",                cmdInsertOrEditTableCommand);
+    commandTable.registerCommand("cmd_editTable",            bgEditTableCommand);
+    commandTable.registerCommand("cmd_SelectTable",          bgSelectTableCommand);
+    commandTable.registerCommand("cmd_SelectRow",            bgSelectTableRowCommand);
+    commandTable.registerCommand("cmd_SelectColumn",         bgSelectTableColumnCommand);
+    commandTable.registerCommand("cmd_SelectCell",           bgSelectTableCellCommand);
+    commandTable.registerCommand("cmd_SelectAllCells",       bgSelectAllTableCellsCommand);
+    commandTable.registerCommand("cmd_InsertTable",          bgInsertTableCommand);
+    commandTable.registerCommand("cmd_InsertRowAbove",       bgInsertTableRowAboveCommand);
+    commandTable.registerCommand("cmd_InsertRowBelow",       bgInsertTableRowBelowCommand);
+    commandTable.registerCommand("cmd_InsertColumnBefore",   bgInsertTableColumnBeforeCommand);
+    commandTable.registerCommand("cmd_InsertColumnAfter",    bgInsertTableColumnAfterCommand);
+    commandTable.registerCommand("cmd_InsertCellBefore",     bgInsertTableCellBeforeCommand);
+    commandTable.registerCommand("cmd_InsertCellAfter",      bgInsertTableCellAfterCommand);
+    commandTable.registerCommand("cmd_DeleteTable",          bgDeleteTableCommand);
+    commandTable.registerCommand("cmd_DeleteRow",            bgDeleteTableRowCommand);
+    commandTable.registerCommand("cmd_DeleteColumn",         bgDeleteTableColumnCommand);
+    commandTable.registerCommand("cmd_DeleteCell",           bgDeleteTableCellCommand);
+    commandTable.registerCommand("cmd_DeleteCellContents",   bgDeleteTableCellContentsCommand);
+    commandTable.registerCommand("cmd_JoinTableCells",       bgJoinTableCellsCommand);
+    commandTable.registerCommand("cmd_SplitTableCell",       bgSplitTableCellCommand);
+    commandTable.registerCommand("cmd_NormalizeTable",       bgNormalizeTableCommand);
+    commandTable.registerCommand("cmd_ConvertToTable",       bgConvertToTable);
 
     commandTable.registerCommand("cmd_image",       cmdInsertImageCommand);
     commandTable.registerCommand("cmd_anchor",      cmdInsertAnchorCommand);
@@ -365,11 +384,19 @@ var ComposerCommands = {
     commandTable.registerCommand("cmd_ariaRole",    cmdAriaRole);
 
     commandTable.registerCommand("cmd_bgfind",      cmdBgFindCommand);
+    commandTable.registerCommand("cmd_replace",     cmdBgFindCommand);
 
     commandTable.registerCommand("cmd_stylesheets", cmdStylesheetsCommand);
 
     commandTable.registerCommand("cmd_span",        cmdSpanCommand);
-  },
+
+    commandTable.registerCommand("cmd_pageProperties", cmdPagePropertiesCommand);
+
+    commandTable.registerCommand("cmd_spellCheck",  bgSpellingCommand);
+
+    commandTable.registerCommand("cmd_copyHTMLCode", cmdCopyHTMLCodeCommand);
+    commandTable.registerCommand("cmd_gotoLink",    cmdGotoLinkCommand);
+},
 
   setupFormatCommands: function setupFormatCommands()
   {
@@ -437,10 +464,10 @@ var ComposerCommands = {
     }
 
     if (ComposerCommands.mLastSelectedElementPath != path) {
-	    // now we're sure something changed in the selection, element or attribute
-	    // on the selected element
+      // now we're sure something changed in the selection, element or attribute
+      // on the selected element
       if (!aDontNotify)
-  	    NotifierUtils.notify("selection", aElement, aOneElementSelected);
+        NotifierUtils.notify("selection", aElement, aOneElementSelected);
       ComposerCommands.mLastSelectedElementPath = path;
     }
   },
@@ -577,19 +604,20 @@ var cmdSave =
     var result = false;
     var editor = EditorUtils.getCurrentEditor();
     if (editor)
-    try {
-      ToggleViewMode(gDialog.wysiwygModeButton);
-      // for the time being, we use the HTML serializer
-
-      result = FileUtils.saveDocument(
-                 UrlUtils.isUrlOfBlankDocument(EditorUtils.getDocumentUrl()),
-                 false,
-                 EditorUtils.getCurrentDocumentMimeType());
-      window.content.focus();
-      if (result)
-        StoreUrlInLocationDB(EditorUtils.getDocumentUrl());
-    }
-    catch (e) {}
+      try {
+        if (gDialog.wysiwygModeButton)
+          ToggleViewMode(gDialog.wysiwygModeButton);
+        // for the time being, we use the HTML serializer
+  
+        result = FileUtils.saveDocument(
+                   UrlUtils.isUrlOfBlankDocument(EditorUtils.getDocumentUrl()),
+                   false,
+                   EditorUtils.getCurrentDocumentMimeType());
+        window.content.focus();
+        if (result)
+          StoreUrlInLocationDB(EditorUtils.getDocumentUrl());
+      }
+      catch (e) {}
     return result;
   }
 }
@@ -609,7 +637,8 @@ var cmdSaveAs =
     var editor = EditorUtils.getCurrentEditor();
     if (editor)
     try {
-      ToggleViewMode(gDialog.wysiwygModeButton);
+      if (gDialog.wysiwygModeButton)
+        ToggleViewMode(gDialog.wysiwygModeButton);
       var editorMimeType = EditorUtils.getCurrentDocument().contentType;
       var result = FileUtils.saveDocument(true, false, editorMimeType);
       if (result)
@@ -762,12 +791,32 @@ var cmdDummyHTML =
   doCommand: function(aCommand)
   {
     // do nothing
-    dump("Hey, who's calling the dummy command?\n");
   }
 
 };
 
-var cmdDummy2HTML =
+var cmdDummySource =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    return (EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            !EditorUtils.activeViewActive  &&
+            GetCurrentViewMode() == "source");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    // do nothing
+  }
+
+};
+
+var cmdDummyAll =
 {
   isCommandEnabled: function(aCommand, dummy)
   {
@@ -783,7 +832,27 @@ var cmdDummy2HTML =
   doCommand: function(aCommand)
   {
     // do nothing
-    dump("Hey, who's calling the dummy command?\n");
+  }
+
+};
+
+var cmdViewModeEnabler =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    return (EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            !EditorUtils.activeViewActive &&
+            gDialog.sourceModeButton.getAttribute("busy") != "true");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    // do nothing
   }
 };
 
@@ -808,6 +877,45 @@ var cmdMarkupCleaner =
     window.content.focus();
   }
 };
+
+var cmdGotoLinkCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    if (EditorUtils.getCurrentEditorElement() &&
+        EditorUtils.isDocumentEditable() &&
+        EditorUtils.isEditingRenderedHTML() &&
+        GetCurrentViewMode() == "wysiwyg") {
+      var element = EditorUtils.getObjectForProperties(["a"], "href");
+      if (element) {
+        var url = UrlUtils.makeRelativeUrl(element.getAttribute("href"));
+        if (url.length && url[0] == "#")
+          return true;
+      }
+    }
+    return false;
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    var element = EditorUtils.getObjectForProperties(["a"], "href");
+    var target = UrlUtils.makeRelativeUrl(element.getAttribute("href"));
+    if (target)
+      target = target.substr(1);
+    else
+      return; // sanity check
+    // first check ID...
+    var doc = EditorUtils.getCurrentDocument();
+    var targetElement = doc.getElementById(target) || doc.querySelector("a[name='" + target + "']");
+    if (targetElement) {
+      ScrollToElement(targetElement);
+      EditorUtils.getCurrentEditor().selectElement(targetElement);
+    }
+  }
+};
 //-----------------------------------------------------------------------------------
 
 var cmdClass =
@@ -980,6 +1088,29 @@ var cmdAriaRole =
   }
 };
 
+var cmdPagePropertiesCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    return (EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      window.openDialog("chrome://bluegriffon/content/dialogs/pageProperties.xul", "_blank",
+              "chrome,close,titlebar,modal");
+    }
+    catch(ex) {}
+    window.content.focus();
+  }
+};
 
 var cmdInsertOrEditTableCommand =
 {
@@ -997,7 +1128,7 @@ var cmdInsertOrEditTableCommand =
   doCommand: function(aCommand)
   {
     var element = EditorUtils.getObjectForProperties(
-          ["table", "thead", "tfoot", "tbody", "tr", "th", "td", "cpation"]);
+          ["table", "thead", "tfoot", "tbody", "tr", "th", "td", "caption"]);
     if (element) {
       OpenAppModalWindow(window, "chrome://bluegriffon/content/dialogs/insertTable.xul", "Tables", false, element); 
       return;
@@ -1502,10 +1633,859 @@ var cmdBgFindCommand =
 
   doCommand: function(aCommand)
   {
-    if (GetCurrentViewMode() == "source")
-      gDialog.bespinFindTextbox.focus();
+    var isSource = (GetCurrentViewMode() == "source");
+    WysiwygShowFindBar();
+    gDialog.bespinLineLabel.hidden = !isSource;
+    gDialog.bespinLineTextbox.hidden = !isSource;
   }
 };
 
-//@line 476 "/home/glazou/trees/bg-trunk/bluegriffon/base/content/bluegriffon/js/commands.js"
+var bgSpellingCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    return (EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    window.cancelSendMessage = false;
+    try {
+      OpenAppModalWindow(window,
+                         "chrome://bluegriffon/content/dialogs/spellCheck.xul",
+                         "Spell Check",
+                         false); 
+    }
+    catch(ex) {}
+    window.content.focus();
+  }
+};
+
+var cmdCopyHTMLCodeCommand = {
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var isHTMLView = (EditorUtils.getCurrentEditorElement() &&
+                      EditorUtils.isDocumentEditable() &&
+                      EditorUtils.isEditingRenderedHTML() &&
+                      GetCurrentViewMode() == "wysiwyg");
+    if (isHTMLView) {
+      var editor = EditorUtils.getCurrentEditor();
+      var selection = editor.selection;
+      if (selection.rangeCount == 1)
+      {
+        // We have a "normal" single-range selection
+        if (!selection.isCollapsed) {
+           return true;
+         }
+      }
+    }
+    return false;
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    var editor    = EditorUtils.getCurrentEditor();
+    var selection = editor.selection;
+    selection instanceof Components.interfaces.nsISelectionPrivate;
+    var mimeType = EditorUtils.getCurrentDocument().contentType;
+
+    var flags = 1 << 1; // OutputFormatted
+    flags |= 1 << 5; // OutputWrap
+    flags |= 1 << 10; // OutputLF
+
+    var source = selection.toStringWithFormat(mimeType,
+                                              flags,
+                                              0);
+
+    var clipboardSvc = Components.classes["@mozilla.org/widget/clipboard;1"]
+                         .getService(Components.interfaces.nsIClipboard);
+    var xferable = Components.classes["@mozilla.org/widget/transferable;1"]
+                     .createInstance(Components.interfaces.nsITransferable);
+    xferable.addDataFlavor("text/unicode");
+    var s = Components.classes["@mozilla.org/supports-string;1"]
+              .createInstance(Components.interfaces.nsISupportsString);
+    s.data = source;
+    xferable.setTransferData("text/unicode", s, source.length * 2);
+    clipboardSvc.setData(xferable, null, Components.interfaces.nsIClipboard.kGlobalClipboard);
+  }
+};
+
+var bgInsertTableCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    return (EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    // reset the table insertion panel
+    AutoInsertTable.reset();
+    gDialog.AutoInsertTableSheetPopup.openPopup(gDialog["tableButton"], "after_start", 0, 0, false);
+  }
+};
+
+var bgEditTableCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["table"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    var element = EditorUtils.getObjectForProperties(["table"]);
+    OpenAppModalWindow(window, "chrome://bluegriffon/content/dialogs/insertTable.xul", "Tables", false, element); 
+  }
+};
+
+var bgSelectTableCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["table"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().selectTable();
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgSelectTableRowCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().selectTableRow();
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgsSelectTableColumnCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().selectTableColumn();
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgSelectTableCellCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().selectTableCell();
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgSelectAllTableCellsCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["table"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().selectAllTableCells();
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgInsertTableRowAboveCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().insertTableRow(1, false);
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgInsertTableRowBelowCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().insertTableRow(1, true);
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgInsertTableColumnBeforeCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().insertTableColumn(1, false);
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgInsertTableColumnAfterCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().insertTableColumn(1, true);
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgInsertTableCellBeforeCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().insertTableCell(1, false);
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgInsertTableCellAfterCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().insertTableCell(1, true);
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+var bgDeleteTableCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["table"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().deleteTable();
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+function GetNumberOfContiguousSelectedRows()
+{
+  var rows = 0;
+  try {
+    var editor = EditorUtils.getCurrentTableEditor();
+    var rowObj = { value: 0 };
+    var colObj = { value: 0 };
+    var cell = editor.getFirstSelectedCellInTable(rowObj, colObj);
+    if (!cell)
+      return 0;
+
+    // We have at least one row
+    rows++;
+
+    var lastIndex = rowObj.value;
+    do {
+      cell = editor.getNextSelectedCell({value:0});
+      if (cell)
+      {
+        editor.getCellIndexes(cell, rowObj, colObj);
+        var index = rowObj.value;
+        if (index == lastIndex + 1)
+        {
+          lastIndex = index;
+          rows++;
+        }
+      }
+    }
+    while (cell);
+  } catch (e) {}
+
+  return rows;
+}
+
+function GetNumberOfContiguousSelectedColumns()
+{
+  var columns = 0;
+  try {
+    var editor = EditorUtils.getCurrentTableEditor();
+    var colObj = { value: 0 };
+    var rowObj = { value: 0 };
+    var cell = editor.getFirstSelectedCellInTable(rowObj, colObj);
+    if (!cell)
+      return 0;
+
+    // We have at least one column
+    columns++;
+
+    var lastIndex = colObj.value;
+    do {
+      cell = editor.getNextSelectedCell({value:0});
+      if (cell)
+      {
+        editor.getCellIndexes(cell, rowObj, colObj);
+        var index = colObj.value;
+        if (index == lastIndex +1)
+        {
+          lastIndex = index;
+          columns++;
+        }
+      }
+    }
+    while (cell);
+  } catch (e) {}
+
+  return columns;
+}
+
+var bgDeleteTableRowCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    var rows = GetNumberOfContiguousSelectedRows();
+    // Delete at least one row
+    if (rows == 0)
+      rows = 1;
+
+    try {
+      var editor = EditorUtils.getCurrentTableEditor();
+      editor.beginTransaction();
+
+      // Loop to delete all blocks of contiguous, selected rows
+      while (rows)
+      {
+        editor.deleteTableRow(rows);
+        rows = GetNumberOfContiguousSelectedRows();
+      }
+    } finally { editor.endTransaction(); }
+    window.content.focus();
+  }
+};
+
+var bgDeleteTableColumnCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    var columns = GetNumberOfContiguousSelectedColumns();
+    // Delete at least one column
+    if (columns == 0)
+      columns = 1;
+
+    try {
+      var editor = EditorUtils.getCurrentTableEditor();
+      editor.beginTransaction();
+
+      // Loop to delete all blocks of contiguous, selected columns
+      while (columns)
+      {
+        editor.deleteTableColumn(columns);
+        columns = GetNumberOfContiguousSelectedColumns();
+      }
+    } finally { editor.endTransaction(); }
+    window.content.focus();
+  }
+};
+
+var bgDeleteTableCellCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().deleteTableCell(1);   
+    } catch (e) {}
+    window.content.focus();
+  }
+};
+
+var bgDeleteTableCellContentsCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().deleteTableCellContents();   
+    } catch (e) {}
+    window.content.focus();
+  }
+};
+
+var bgNormalizeTableCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["table"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    // Use nsnull to let editor find table enclosing current selection
+    try {
+      EditorUtils.getCurrentTableEditor().normalizeTable(null);   
+    } catch (e) {}
+    window.content.focus();
+  }
+};
+
+var bgJoinTableCellsCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    if (EditorUtils.getCurrentEditorElement() &&
+        EditorUtils.isDocumentEditable() &&
+        EditorUtils.isEditingRenderedHTML() &&
+        GetCurrentViewMode() == "wysiwyg")
+    {
+      try {
+        var editor = EditorUtils.getCurrentTableEditor();
+        var tagNameObj = { value: "" };
+        var countObj = { value: 0 };
+        var cell = editor.getSelectedOrParentTableElement(tagNameObj, countObj);
+
+        // We need a cell and either > 1 selected cell or a cell to the right
+        //  (this cell may originate in a row spanned from above current row)
+        // Note that editor returns "td" for "th" also.
+        // (this is a pain! Editor and gecko use lowercase tagNames, JS uses uppercase!)
+        if( cell && (tagNameObj.value == "td"))
+        {
+          // Selected cells
+          if (countObj.value > 1) return true;
+
+          var colSpan = cell.getAttribute("colspan");
+
+          // getAttribute returns string, we need number
+          // no attribute means colspan = 1
+          if (!colSpan)
+            colSpan = Number(1);
+          else
+            colSpan = Number(colSpan);
+
+          var rowObj = { value: 0 };
+          var colObj = { value: 0 };
+          editor.getCellIndexes(cell, rowObj, colObj);
+
+          // Test if cell exists to the right of current cell
+          // (cells with 0 span should never have cells to the right
+          //  if there is, user can select the 2 cells to join them)
+          return (colSpan && editor.getCellAt(null, rowObj.value,
+                                              colObj.value + colSpan));
+        }
+      } catch (e) {}
+    }
+    return false;
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    // Param: Don't merge non-contiguous cells
+    try {
+      EditorUtils.getCurrentTableEditor().joinTableCells(false);
+    } catch (e) {}
+    window.content.focus();
+  }
+};
+
+function IsSelectionInOneCell()
+{
+  try {
+    var editor = EditorUtils.getCurrentEditor();
+    var selection = editor.selection;
+
+    if (selection.rangeCount == 1)
+    {
+      // We have a "normal" single-range selection
+      if (!selection.isCollapsed &&
+         selection.anchorNode != selection.focusNode)
+      {
+        // Check if both nodes are within the same cell
+        var anchorCell = editor.getElementOrParentByTagName("td", selection.anchorNode);
+        var focusCell = editor.getElementOrParentByTagName("td", selection.focusNode);
+        return (focusCell != null && anchorCell != null && (focusCell == anchorCell));
+      }
+      // Collapsed selection or anchor == focus (thus must be in 1 cell)
+      return true;
+    }
+  } catch (e) {}
+  return false;
+}
+
+var bgSplitTableCellCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    if (EditorUtils.getCurrentEditorElement() &&
+        EditorUtils.isDocumentEditable() &&
+        EditorUtils.isEditingRenderedHTML() &&
+        GetCurrentViewMode() == "wysiwyg")
+    {
+      var tagNameObj = { value: "" };
+      var countObj = { value: 0 };
+      var cell;
+      try {
+        cell = EditorUtils.getCurrentTableEditor().getSelectedOrParentTableElement(tagNameObj, countObj);
+      } catch (e) {}
+
+      // We need a cell parent and there's just 1 selected cell 
+      // or selection is entirely inside 1 cell
+      if ( cell && (tagNameObj.value == "td") && 
+           countObj.value <= 1 &&
+           IsSelectionInOneCell() )
+      {
+        var colSpan = cell.getAttribute("colspan");
+        var rowSpan = cell.getAttribute("rowspan");
+        if (!colSpan) colSpan = 1;
+        if (!rowSpan) rowSpan = 1;
+        return (colSpan > 1  || rowSpan > 1 ||
+                colSpan == 0 || rowSpan == 0);
+      }
+    }
+    return false;
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().splitTableCell();
+    } catch (e) {}
+    window.content.focus();
+  }
+};
+
+function GetParentTableCell(element)
+{
+  var node = element;
+  while (node)
+  {
+    if (node.nodeName.toLowerCase() == "td" || node.nodeName.toLowerCase() == "th")
+      return node;
+
+    node = node.parentNode;
+  }
+  return node;
+}
+
+var bgConvertToTable =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    if (EditorUtils.getCurrentEditorElement() &&
+        EditorUtils.isDocumentEditable() &&
+        EditorUtils.isEditingRenderedHTML() &&
+        GetCurrentViewMode() == "wysiwyg")
+    {
+      var selection;
+      var editor = EditorUtils.getCurrentEditor();
+      try {
+        selection = editor.selection;
+      } catch (e) {}
+
+      if (selection && !selection.isCollapsed)
+      {
+        // Don't allow if table or cell is the selection
+        var element;
+        try {
+          element = editor.getSelectedElement("");
+        } catch (e) {}
+        if (element)
+        {
+          var name = element.nodeName.toLowerCase();
+          if (name == "td" ||
+              name == "th" ||
+              name == "caption" ||
+              name == "table")
+            return false;
+        }
+
+        // Selection start and end must be in the same cell
+        //   in same cell or both are NOT in a cell
+        if ( GetParentTableCell(selection.focusNode) !=
+             GetParentTableCell(selection.anchorNode) )
+          return false
+      
+        return true;
+      }
+    }
+    return false;
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    window.openDialog("chrome://bluegriffon/content/dialogs/convertToTable.xul","_blank", "chrome,close,titlebar,modal")
+    window.content.focus();
+  }
+};
+
+var bgSelectTableColumnCommand =
+{
+  isCommandEnabled: function(aCommand, dummy)
+  {
+    var element = EditorUtils.getObjectForProperties(["td", "th"]);
+    return (element &&
+            EditorUtils.getCurrentEditorElement() &&
+            EditorUtils.isDocumentEditable() &&
+            EditorUtils.isEditingRenderedHTML() &&
+            GetCurrentViewMode() == "wysiwyg");
+  },
+
+  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
+  doCommandParams: function(aCommand, aParams, aRefCon) {},
+
+  doCommand: function(aCommand)
+  {
+    try {
+      EditorUtils.getCurrentTableEditor().selectTableColumn();
+    } catch(e) {}
+    window.content.focus();
+  }
+};
+
+
+//@line 504 "/home/glazou/trees/bg1/bluegriffon/base/content/bluegriffon/js/commands.js"
 
