@@ -42,9 +42,9 @@ Components.utils.import("resource://app/modules/prompterHelper.jsm");
 
 var gMain = null;
 var gCurrentElement = null;
-//@line 49 "/home/glazou/trees/bg-trunk/bluegriffon/sidebars/domexplorer/content/domexplorer.js"
+//@line 49 "/home/glazou/trees/bg1/bluegriffon/sidebars/domexplorer/content/domexplorer.js"
 var gIsPanelActive = true;
-//@line 54 "/home/glazou/trees/bg-trunk/bluegriffon/sidebars/domexplorer/content/domexplorer.js"
+//@line 54 "/home/glazou/trees/bg1/bluegriffon/sidebars/domexplorer/content/domexplorer.js"
 var gPrefs = null;
 var gPath = null;
 var gNewAttribute = -1;
@@ -91,6 +91,7 @@ function Startup()
       SelectionChanged(null, c.node, c.oneElementSelected);
   }
 
+  gDialog.elementsTree.addEventListener("DOMAttrModified", onElementsTreeModified, true);
   gDialog.attributesTree.addEventListener("DOMAttrModified", onAttributesTreeModified, true);
   gDialog.cssTree.addEventListener("DOMAttrModified", onCssTreeModified, true);
 }
@@ -114,6 +115,7 @@ function Shutdown()
     gMain.NotifierUtils.removeNotifierCallback("panelClosed",
                                                 PanelClosed,
                                                 window);
+    gDialog.elementsTree.removeEventListener("DOMAttrModified", onElementsTreeModified, true);
 	  gDialog.attributesTree.removeEventListener("DOMAttrModified", onAttributesTreeModified, true);
     gDialog.cssTree.removeEventListener("DOMAttrModified", onCssTreeModified, true);
   }
@@ -221,13 +223,32 @@ function SelectionChanged(aArgs, aElt, aOneElementSelected)
       treeitem = tmp;
     }
   }
-  if (selected)
+  if (selected) {
+    var tmp = gDialog.elementsTree.getAttribute("onselect");
+    gDialog.elementsTree.removeAttribute("onselect")
     gDialog.elementsTree.view.selection.select(gDialog.elementsTree.contentView.getIndexOfItem(selected));
+    gDialog.elementsTree.setAttribute("onselect", tmp)
+  }
 
   UpdateAttributes();
   UpdateStyles();
 }
 
+function GetSelectedElementInTree()
+{
+  var tree = gDialog.elementsTree;
+  var contentView = tree.contentView;
+  var view = tree.view;
+  if (!view || !view.selection || !view.selection.count) // No selection yet in the tree
+  {
+    return;
+  }
+  var index = view.selection.currentIndex;
+  var treeitem = contentView.getItemAtIndex(index);
+  var node = treeitem.getUserData("node");
+  return node;
+}
+
 function ElementSelectedInTree()
 {
   var tree = gDialog.elementsTree;
@@ -243,7 +264,11 @@ function ElementSelectedInTree()
 
   gDialog.elementsTree.view.selection.select(index);
   gMain.ComposerCommands.mLastSelectedElement = null;
-  SelectionChanged(null, node, true);
+  gMain.ScrollToElement(node);
+  try {
+    SelectionChanged(null, node, true);
+  }
+  catch(e) {}
 }
 
 function UpdateAttributes()
